==> openturns_1.18.orig.tar.gz: openturns-1.18/lib/test/t_Mixture_std.cxx <==
//                                               -*- C++ -*-
/**
 *  @brief The test file of class Mixture for standard methods
 *
 *  Copyright 2005-2021 Airbus-EDF-IMACS-ONERA-Phimeca
 *
 *  This library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "openturns/OT.hxx"
#include "openturns/OTtestcode.hxx"

using namespace OT;
using namespace OT::Test;

int main(int, char *[])
{
  TESTPREAMBLE;
  OStream fullprint(std::cout);
  setRandomGenerator();

  try
  {

    // First, 1D case
    {
      Mixture::DistributionCollection atoms(3);
      atoms[0] = Normal(-3.0, 1.0);
      atoms[0].setWeight(1.0);
      atoms[1] = Normal(1.0, 2.0);
      atoms[1].setWeight(2.0);
      atoms[2] = Normal(3.0, 3.0);
      atoms[2].setWeight(4.0);
      Mixture distribution(atoms);
      Mixture::PointWithDescriptionCollection parameters(distribution.getParametersCollection());
      fullprint << "parameters=" << parameters << std::endl;
      Point parameter(distribution.getParameter());
      fullprint << "parameter=" << parameter << std::endl;
      Description parameterDescription(distribution.getParameterDescription());
      fullprint << "parameter description=" << parameterDescription << std::endl;
      distribution.setParameter(parameter);

      for (UnsignedInteger i = 0; i < 6; ++i) fullprint << "standard moment n=" << i << ", value=" << distribution.getStandardMoment(i) << std::endl;
      fullprint << "Standard representative=" << distribution.getStandardRepresentative().__str__() << std::endl;
      // get weights used
      Point weights(distribution.getWeights());
      fullprint << "weights=" << weights << std::endl;
      // Fix new weights
      weights[0] = 0.2;
      weights[1] = 0.5;
      //set item
      distribution.setWeights(weights);
      fullprint << "New weights=" << distribution.getWeights() << std::endl;
    }
    UnsignedInteger dimension = 3;
    Point meanPoint(dimension, 1.0);
    meanPoint[0] = 0.5;
    meanPoint[1] = -0.5;
    Point sigma(dimension, 1.0);
    sigma[0] = 2.0;
    sigma[1] = 3.0;
    CorrelationMatrix R(dimension);
    for (UnsignedInteger i = 1; i < dimension; i++)
    {
      R(i, i - 1) = 0.5;
    }
    // Create a collection of distribution
    Mixture::DistributionCollection aCollection;

    aCollection.add( Normal(meanPoint, sigma, R) );
    meanPoint += Point(dimension, 1.0);
    aCollection.add( Normal(meanPoint, sigma, R) );
    meanPoint += Point(dimension, 1.0);
    aCollection.add( Normal(meanPoint, sigma, R) );

    // Instanciate one distribution object
    Mixture distribution(aCollection, Point(aCollection.getSize(), 2.0));
    fullprint << "Distribution " << distribution << std::endl;
    distribution = Mixture(aCollection);
    fullprint << "Distribution " << distribution.__str__() << std::endl;

    // Is this distribution elliptical ?
    fullprint << "Elliptical = " << (distribution.isElliptical() ? "true" : "false") << std::endl;

    // Is this distribution continuous ?
    fullprint << "Continuous = " << (distribution.isContinuous() ? "true" : "false") << std::endl;

    // Test for realization of distribution
    Point oneRealization = distribution.getRealization();
    fullprint << "oneRealization=" << oneRealization << std::endl;

    // Test for sampling
    UnsignedInteger size = 1000;
    Sample oneSample = distribution.getSample( size );
    fullprint << "oneSample first=" << oneSample[0] << " last=" << oneSample[size - 1] << std::endl;
    fullprint << "mean=" << oneSample.computeMean() << std::endl;
    fullprint << "covariance=" << oneSample.computeCovariance() << std::endl;
    if (distribution.getDimension() == 1)
    {
      size = 100;
      for (UnsignedInteger i = 0; i < 2; ++i)
      {
        fullprint << "Kolmogorov test for the generator, sample size=" << size << " is " << (FittingTest::Kolmogorov(distribution.getSample(size), distribution).getBinaryQualityMeasure() ? "accepted" : "rejected") << std::endl;
        size *= 10;
      }
    }
    // Define a point
    Point point( dimension, 1.0 );
    fullprint << "Point= " << point << std::endl;

    // Show PDF and CDF of point
    Scalar eps = 1e-5;
    Point DDF = distribution.computeDDF( point );
    fullprint << "ddf     =" << DDF << std::endl;
    Point ddfFD(dimension);
    for (UnsignedInteger i = 0; i < dimension; i++)
    {
      Point left(point);
      left[i] += eps;
      Point right(point);
      right[i] -= eps;
      ddfFD[i] = (distribution.computePDF(left) - distribution.computePDF(right)) / (2.0 * eps);
    }
    fullprint << "ddf (FD)=" << ddfFD << std::endl;
    Scalar LPDF = distribution.computeLogPDF( point );
    fullprint << "log pdf=" << LPDF << std::endl;
    Scalar PDF = distribution.computePDF( point );
    fullprint << "pdf     =" << PDF << std::endl;
    if (dimension == 1)
    {
      fullprint << "pdf (FD)=" << (distribution.computeCDF( point + Point(1, eps) ) - distribution.computeCDF( point  + Point(1, -eps) )) / (2.0 * eps) << std::endl;
    }
    Scalar CDF = distribution.computeCDF( point );
    fullprint << "cdf=" << CDF << std::endl;
    Scalar CCDF = distribution.computeComplementaryCDF( point );
    fullprint << "ccdf=" << CCDF << std::endl;
    Scalar Survival = distribution.computeSurvivalFunction( point );
    fullprint << "survival =" << Survival << std::endl;
    Point InverseSurvival = distribution.computeInverseSurvivalFunction(0.95);
    fullprint << "Inverse survival=" << InverseSurvival << std::endl;
    fullprint << "Survival(inverse survival)=" << distribution.computeSurvivalFunction(InverseSurvival) << std::endl;
    if (dimension == 1)
    {
      Complex CF = distribution.computeCharacteristicFunction( point[0] );
      fullprint << "characteristic function=" << CF << std::endl;
      Complex LCF = distribution.computeLogCharacteristicFunction( point[0] );
      fullprint << "log characteristic function=" << LCF << std::endl;
    }
    Point quantile = distribution.computeQuantile( 0.95 );
    fullprint << "quantile=" << quantile << std::endl;
    fullprint << "cdf(quantile)=" << distribution.computeCDF(quantile) << std::endl;
    if (distribution.getDimension() <= 2)
    {
      // Confidence regions
      Scalar threshold;
      fullprint << "Minimum volume interval=" << distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95, threshold) << std::endl;
      fullprint << "threshold=" << threshold << std::endl;
      Scalar beta;
      LevelSet levelSet(distribution.computeMinimumVolumeLevelSetWithThreshold(0.95, beta));
      fullprint << "Minimum volume level set=" << levelSet << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Bilateral confidence interval=" << distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Unilateral confidence interval (lower tail)=" << distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, false, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Unilateral confidence interval (upper tail)=" << distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, true, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
    }
    // Takes too much time. Exact result is 6.0593
    // fullprint << "entropy=" << distribution.computeEntropy() << std::endl;
    // fullprint << "entropy (MC)=" << -distribution.computeLogPDF(distribution.getSample(1000000)).computeMean()[0] << std::endl;
    Point mean = distribution.getMean();
    fullprint << "mean=" << mean << std::endl;
    CovarianceMatrix covariance = distribution.getCovariance();
    fullprint << "covariance=" << covariance << std::endl;
    CovarianceMatrix correlation = distribution.getCorrelation();
    fullprint << "correlation=" << correlation << std::endl;
    /*    CovarianceMatrix spearman = distribution.getSpearmanCorrelation();
    fullprint << "spearman=" << spearman << std::endl;
    CovarianceMatrix kendall = distribution.getKendallTau();
    fullprint << "kendall=" << kendall << std::endl;*/
    Mixture::PointWithDescriptionCollection parameters = distribution.getParametersCollection();
    fullprint << "parameters=" << parameters << std::endl;

    Scalar x = 0.6;
    Point y(1, 0.2);
    fullprint << "conditional PDF=" << distribution.computeConditionalPDF(x, y) << std::endl;
    fullprint << "conditional CDF=" << distribution.computeConditionalCDF(x, y) << std::endl;
    fullprint << "conditional quantile=" << distribution.computeConditionalQuantile(x, y) << std::endl;
    Point pt(distribution.getDimension());
    for (UnsignedInteger i = 0; i < distribution.getDimension(); ++i) pt[i] = 1.0 * i + 1.5;
    fullprint << "sequential conditional PDF=" << distribution.computeSequentialConditionalPDF(pt) << std::endl;
    Point resCDF(distribution.computeSequentialConditionalCDF(pt));
    fullprint << "sequential conditional CDF(" << pt << ")=" << resCDF << std::endl;
    fullprint << "sequential conditional quantile(" << resCDF << ")=" << distribution.computeSequentialConditionalQuantile(resCDF) << std::endl;

    // Constructor with separate weights. Also check small weights removal
    Point weights;
    Mixture::DistributionCollection atoms;
    weights.add(1.0e-20);
    atoms.add(Normal(1.0, 1.0));
    weights.add(2.5);
    atoms.add(Normal(2.0, 2.0));
    weights.add(32.0);
    atoms.add(Normal(3.0, 3.0));
    Mixture newMixture(atoms, weights);
    fullprint << "newMixture pdf=" << newMixture.computePDF(2.5) << std::endl;
    fullprint << "atoms kept in mixture=" << newMixture.getDistributionCollection() << std::endl;
    fullprint << "newMixture=" << newMixture.__str__() << std::endl;



  }
  catch (TestFailed & ex)
  {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }


  return ExitCode::Success;
}

==> openturns_1.25.1.orig.tar.gz: openturns-1.25.1/lib/test/t_Mixture_std.cxx <==
//                                               -*- C++ -*-
/**
 *  @brief The test file of class Mixture for standard methods
 *
 *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
 *
 *  This library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "openturns/OT.hxx"
#include "openturns/OTtestcode.hxx"

using namespace OT;
using namespace OT::Test;

int main(int, char *[])
{
  TESTPREAMBLE;
  OStream fullprint(std::cout);

  try
  {

    // First, 1D case
    {
      Mixture::DistributionCollection atoms(3);
      atoms[0] = Normal(-3.0, 1.0);
      atoms[0].setWeight(1.0);
      atoms[1] = Normal(1.0, 2.0);
      atoms[1].setWeight(2.0);
      atoms[2] = Normal(3.0, 3.0);
      atoms[2].setWeight(4.0);
      Mixture distribution(atoms);
      Mixture::PointWithDescriptionCollection parameters(distribution.getParametersCollection());
      fullprint << "parameters=" << parameters << std::endl;
      Point parameter(distribution.getParameter());
      fullprint << "parameter=" << parameter << std::endl;
      Description parameterDescription(distribution.getParameterDescription());
      fullprint << "parameter description=" << parameterDescription << std::endl;
      distribution.setParameter(parameter);

      fullprint << "Standard representative=" << distribution.getStandardRepresentative().__str__() << std::endl;
      // get weights used
      Point weights(distribution.getWeights());
      fullprint << "weights=" << weights << std::endl;
      // Fix new weights
      weights[0] = 0.2;
      weights[1] = 0.5;
      //set item
      distribution.setWeights(weights);
      fullprint << "New weights=" << distribution.getWeights() << std::endl;
    }
    UnsignedInteger dimension = 3;
    Point meanPoint(dimension, 1.0);
    meanPoint[0] = 0.5;
    meanPoint[1] = -0.5;
    Point sigma(dimension, 1.0);
    sigma[0] = 2.0;
    sigma[1] = 3.0;
    CorrelationMatrix R(dimension);
    for (UnsignedInteger i = 1; i < dimension; i++)
    {
      R(i, i - 1) = 0.5;
    }
    // Create a collection of distribution
    Mixture::DistributionCollection aCollection;

    aCollection.add( Normal(meanPoint, sigma, R) );
    meanPoint += Point(dimension, 1.0);
    aCollection.add( Normal(meanPoint, sigma, R) );
    meanPoint += Point(dimension, 1.0);
    aCollection.add( Normal(meanPoint, sigma, R) );

    // Instantiate one distribution object
    Mixture distribution(aCollection, Point(aCollection.getSize(), 2.0));
    fullprint << "Distribution " << distribution << std::endl;
    distribution = Mixture(aCollection);
    fullprint << "Distribution " << distribution.__str__() << std::endl;

    // Is this distribution elliptical ?
    fullprint << "Elliptical = " << (distribution.isElliptical() ? "true" : "false") << std::endl;

    // Is this distribution continuous ?
    fullprint << "Continuous = " << (distribution.isContinuous() ? "true" : "false") << std::endl;

    // Test for realization of distribution
    Point oneRealization = distribution.getRealization();
    fullprint << "oneRealization=" << oneRealization << std::endl;

    // Test for sampling
    UnsignedInteger size = 1000;
    Sample oneSample = distribution.getSample( size );
    fullprint << "oneSample first=" << oneSample[0] << " last=" << oneSample[size - 1] << std::endl;
    fullprint << "mean=" << oneSample.computeMean() << std::endl;
    fullprint << "covariance=" << oneSample.computeCovariance() << std::endl;
    if (distribution.getDimension() == 1)
    {
      size = 100;
      for (UnsignedInteger i = 0; i < 2; ++i)
      {
        fullprint << "Kolmogorov test for the generator, sample size=" << size << " is " << (FittingTest::Kolmogorov(distribution.getSample(size), distribution).getBinaryQualityMeasure() ? "accepted" : "rejected") << std::endl;
        size *= 10;
      }
    }
    // Define a point
    Point point( dimension, 1.0 );
    fullprint << "Point= " << point << std::endl;

    // Show PDF and CDF of point
    Scalar eps = 1e-5;
    Point DDF = distribution.computeDDF( point );
    fullprint << "ddf     =" << DDF << std::endl;
    Point ddfFD(dimension);
    for (UnsignedInteger i = 0; i < dimension; i++)
    {
      Point left(point);
      left[i] += eps;
      Point right(point);
      right[i] -= eps;
      ddfFD[i] = (distribution.computePDF(left) - distribution.computePDF(right)) / (2.0 * eps);
    }
    fullprint << "ddf (FD)=" << ddfFD << std::endl;
    Scalar LPDF = distribution.computeLogPDF( point );
    fullprint << "log pdf=" << LPDF << std::endl;
    Scalar PDF = distribution.computePDF( point );
    fullprint << "pdf     =" << PDF << std::endl;
    if (dimension == 1)
    {
      fullprint << "pdf (FD)=" << (distribution.computeCDF( point + Point(1, eps) ) - distribution.computeCDF( point  + Point(1, -eps) )) / (2.0 * eps) << std::endl;
    }
    Scalar CDF = distribution.computeCDF( point );
    fullprint << "cdf=" << CDF << std::endl;
    Scalar CCDF = distribution.computeComplementaryCDF( point );
    fullprint << "ccdf=" << CCDF << std::endl;
    Scalar Survival = distribution.computeSurvivalFunction( point );
    fullprint << "survival =" << Survival << std::endl;
    Point InverseSurvival = distribution.computeInverseSurvivalFunction(0.95);
    fullprint << "Inverse survival=" << InverseSurvival << std::endl;
    fullprint << "Survival(inverse survival)=" << distribution.computeSurvivalFunction(InverseSurvival) << std::endl;
    if (dimension == 1)
    {
      Complex CF = distribution.computeCharacteristicFunction( point[0] );
      fullprint << "characteristic function=" << CF << std::endl;
      Complex LCF = distribution.computeLogCharacteristicFunction( point[0] );
      fullprint << "log characteristic function=" << LCF << std::endl;
    }
    Point quantile = distribution.computeQuantile( 0.95 );
    fullprint << "quantile=" << quantile << std::endl;
    fullprint << "cdf(quantile)=" << distribution.computeCDF(quantile) << std::endl;
    if (distribution.getDimension() <= 2)
    {
      // Confidence regions
      Scalar threshold;
      fullprint << "Minimum volume interval=" << distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95, threshold) << std::endl;
      fullprint << "threshold=" << threshold << std::endl;
      Scalar beta;
      LevelSet levelSet(distribution.computeMinimumVolumeLevelSetWithThreshold(0.95, beta));
      fullprint << "Minimum volume level set=" << levelSet << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Bilateral confidence interval=" << distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Unilateral confidence interval (lower tail)=" << distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, false, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
      fullprint << "Unilateral confidence interval (upper tail)=" << distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, true, beta) << std::endl;
      fullprint << "beta=" << beta << std::endl;
    }
    // Takes too much time. Exact result is 6.0593
    // fullprint << "entropy=" << distribution.computeEntropy() << std::endl;
    // fullprint << "entropy (MC)=" << -distribution.computeLogPDF(distribution.getSample(1000000)).computeMean()[0] << std::endl;
    Point mean = distribution.getMean();
    fullprint << "mean=" << mean << std::endl;
    CovarianceMatrix covariance = distribution.getCovariance();
    fullprint << "covariance=" << covariance << std::endl;
    CovarianceMatrix correlation = distribution.getCorrelation();
    fullprint << "correlation=" << correlation << std::endl;
    /*    CovarianceMatrix spearman = distribution.getSpearmanCorrelation();
    fullprint << "spearman=" << spearman << std::endl;
    CovarianceMatrix kendall = distribution.getKendallTau();
    fullprint << "kendall=" << kendall << std::endl;*/
    Mixture::PointWithDescriptionCollection parameters = distribution.getParametersCollection();
    fullprint << "parameters=" << parameters << std::endl;

    Scalar x = 0.6;
    Point y(1, 0.2);
    fullprint << "conditional PDF=" << distribution.computeConditionalPDF(x, y) << std::endl;
    fullprint << "conditional CDF=" << distribution.computeConditionalCDF(x, y) << std::endl;
    fullprint << "conditional quantile=" << distribution.computeConditionalQuantile(x, y) << std::endl;
    Point pt(distribution.getDimension());
    for (UnsignedInteger i = 0; i < distribution.getDimension(); ++i) pt[i] = 1.0 * i + 1.5;
    fullprint << "sequential conditional PDF=" << distribution.computeSequentialConditionalPDF(pt) << std::endl;
    Point resCDF(distribution.computeSequentialConditionalCDF(pt));
    fullprint << "sequential conditional CDF(" << pt << ")=" << resCDF << std::endl;
    fullprint << "sequential conditional quantile(" << resCDF << ")=" << distribution.computeSequentialConditionalQuantile(resCDF) << std::endl;

    // Constructor with separate weights. Also check small weights removal
    Point weights;
    Mixture::DistributionCollection atoms;
    weights.add(1.0e-20);
    atoms.add(Normal(1.0, 1.0));
    weights.add(2.5);
    atoms.add(Normal(2.0, 2.0));
    weights.add(32.0);
    atoms.add(Normal(3.0, 3.0));
    Mixture newMixture(atoms, weights);
    fullprint << "newMixture pdf=" << newMixture.computePDF(2.5) << std::endl;
    fullprint << "atoms kept in mixture=" << newMixture.getDistributionCollection() << std::endl;
    fullprint << "newMixture=" << newMixture.__str__() << std::endl;



  }
  catch (TestFailed & ex)
  {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }


  return ExitCode::Success;
}
